//----------------------------------------------------------------------------

// RC-5

//----------------------------------------------------------------------------

//   :    
//    0.
// ,     (  
//    ),    
//   0     
// T_SAMPLE / 2.    0   
//    .      Sample.
//        
// .       
//       ,
//    PreVal.   ,
//  ,    .
//   ,     
//    T_SAMPLE   
//( ).      Rc5Code.
//      BitCounter.
//  RC5_LENGTH ,  ,  
//   SysVar,    -   ComVar.
//  Extended RC-5 Code,   
//   F (Field).  F  
//  ()   ,
//    .

//----------------------------------------------------------------------------

#include "Main.h"
#include "RC5.h"

//----------------------------- : -----------------------------------

#define RC5_SYS      0x1e //   RC-5
#define PRE            64 //  0
#define RC5_SLOT     1778 //  RC-5, 
#define RC5_LENGTH     14 //  

#define T_SAMPLE ((char)(RC5_SLOT / 2 * F_CLK / PRE + 0.5))

#define CONTROL           //  
//#define EXTENDED          //  RC-5

//----------------------------- : ----------------------------------

static char Sample;         // 
static bool Val;            //  
static bool PreVal;         //  -
static int RC5Code;         // 
static char BitCounter;     //  
volatile char ComVar;       // 

//--------------------------  : ------------------------------

#pragma vector = INT0_vect
__interrupt void EdgeIR(void);   //   
#pragma vector = TIM0_OVF_vect
__interrupt void TimerIR(void);  //  0

//-----------------    RC-5: ----------------------

void RC5_Init(void)
{
  BitCounter = RC5_LENGTH;  //  
  PreVal = 1;               //    
  ComVar = CMD_NOP;         //  
  TCCR0B = (1 << CS00) | (1 << CS01); // CK/64   0
  MCUCR = (1 << SE) | (1 << SM1); //INT0   , sleep enable
  GIFR = (1 << INTF0);      //  
  GIMSK |= (1 << INT0);     // INT0
}

//-------------     : --------------

#pragma vector = INT0_vect
__interrupt void EdgeIR(void)
{
  GIMSK &= ~(1 << INT0);      // INT0
  MCUCR &= ~(1 << SE);        //sleep disable
  PLLCSR = (1 << PLLE) | (1 << PCKE); //enable PLL
  TCNT0 = 256 - T_SAMPLE / 2; //   
  TIFR = (1 << TOV0);         //  
  TIMSK |= (1 << TOIE0);      //   0
  Sample = 2;                 //     
  Val = 0;                    //  
}

//------------------    0: ------------------------

#pragma vector = TIM0_OVF_vect
__interrupt void TimerIR(void)
{
  if(Sample)                        //    
  {
    Val = Pin_RC5;                  //  
    if(--Sample)
    {
      //    
      TCNT0 = 256 - T_SAMPLE;       //    
      if(Val != PreVal)             //   
      {
        RC5Code <<= 1;              //  
        if(!Val) RC5Code |= 1;      //  = 0,  = 1
        return;
      }
    }
    else
    {
      //    
      TCNT0 = 256 - T_SAMPLE;       //  
      PreVal = Val;                 //   
      if(PreVal)                    // ,
      {
        MCUCR &= ~(1 << ISC00);     //INT0  ,
        MCUCR |= (1 << ISC01);
      }
      else
      {
        MCUCR |= (1 << ISC01) | (1 << ISC00); // INT0  
      }
      GIMSK |= (1 << INT0);         // INT0
      if(--BitCounter)              //  
        return;                     //    
      char Sys = (RC5Code >> 6) & 0x1F; // 
      if(Sys == RC5_SYS)
      {
        ComVar = RC5Code & 0x3F;    // 
#ifdef CONTROL
        if(RC5Code & 0x800)
          ComVar |= RC5_CTRL;       //  
#endif
      }
#ifdef EXTENDED
      if(!(RC5Code & 0x1000))       //  F
        ComVar |= 0x40;
#endif
    }
  }
  //   
  BitCounter = RC5_LENGTH;          //  
  PreVal = 1;                       //    
  TIMSK &= ~(1 << TOIE0);           //   0
  MCUCR &= ~((1 << ISC01) | (1 << ISC00)); //INT0   
  //if(ComVar != CMD_NOP)           //TODO
  MCUCR |= (1 << SE);               //sleep enable
  GIMSK |= (1 << INT0);             // INT0
}

//--------------------------   : ----------------------------

char RC5_GetCom(void)
{
  char cmd = ComVar;
  if(cmd != CMD_NOP)
    ComVar = CMD_NOP;
  return(cmd);
}

//----------------------------------------------------------------------------
